/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

using System;
using System.Collections.Generic ;
using org.apache.qpid.transport.codec ;

namespace org.apache.qpid.console
{
	
	/**
	 * Metadata that describes the mapping of an object property to a QMF Object
	 */		
	public class SchemaProperty : SchemaVariable 
	{	
		public int Access {get;set;}
		public bool Index {get;set;}	
		public bool Optional {get;set;}		
				
		public SchemaProperty(IDecoder dec)
		{
			Dictionary<string, Object> map = dec.ReadMap() ;
			base.PopulateData(map) ;
			Name = (string)  map["name"] ;
			
			if (map.ContainsKey("optional")) {
				//System.Console.WriteLine("optional") ;
				Optional = (int)map["optional"] != 0 ;		
			} 
			if (map.ContainsKey("index")) {
				//System.Console.WriteLine("index") ;			
				Index = (int)map["index"] != 0 ;		
			}
			if (map.ContainsKey("access")) {
				//System.Console.WriteLine("access") ;			
				Access = (int) map["access"] ;			
			}
		
		}
	}
}
